signature PO =
  sig 
    eqtype elem
    val le : elem* elem -> bool
  end

signature Mappable = 
  sig 
    eqtype key
    type value
  end  

signature Map = 
  sig
    include Mappable 
    type map

    val empty : map
    val get : map-> key-> value option
    val upd : map-> key* value-> map
  end

functor AssocMap(X: Mappable) : Map = 
  struct 
    type key = X.key
    type value = X.value
    type map = (key* value) list

    val empty= []
    fun get m k  = Option.map (#2) (List.find (fn (k0, _)=> k = k0) m)
    fun upd m (k, v) = (k, v):: (List.filter (fn (k0, _)=> k <> k0) m)
  end

signature OrderedMappable = 
  sig 
    include Mappable
    include PO sharing type elem = key
  end

functor TreeMap(X: OrderedMappable) : Map = ?

signature Hashable = 
  sig
    include OrderedMappable
    val  hash : value -> key
  end

functor HashMap(Hash: Hashable): 
  sig include Map
      sharing type key = Hash.key
      sharing type value = Hash.value
  end = struct end
