signature PO = 
  sig 
    type elem
    val le : elem* elem -> bool
  end

signature SORT = 
  sig
    structure Elements : PO
    datatype seq = empty | cons of Elements.elem * seq 
    val sort : seq-> seq 
  end

functor Sort(X: PO): SORT = 
  struct
    structure Elements = X
    datatype seq = empty | cons of Elements.elem * seq 
    fun insert(a, empty) = cons(a, empty)
      | insert(a, cons(b, s)) = 
          if Elements.le(a, b) then cons(a, cons(b, s))
                               else cons(b, insert(a, s))
    fun sort empty = empty
      | sort (cons(a,s)) = insert(a, sort s)
  end


structure IntPO = 
  struct 
    type elem = int
    val  le = op <=
  end

structure IntSort = Sort(IntPO)

functor LexPO(structure X : PO
              structure Y : PO) : PO =
  struct
    type elem = (X.elem* Y.elem)
         fun le((x1, y1), (x2, y2)) = 
           if X.le(x1, x2) 
           then if X.le(x2, x1) then Y.le(y1, y2) else true
           else false
  end

structure BoolPO = 
  struct
    type elem = bool
    fun le(true, true) = true
      | le(true, false) = false
      | le(false, b) = true
  end

structure BoolIntPO = LexPO(structure X= BoolPO structure Y= IntPO)

structure FunnySort = Sort(BoolIntPO)